;*********************************************************************************
;                                                                     
;	Software RS232 communication
;	============================
;
;	Writen by: Andric Dragan, 21.07.1999, mikroElektronika
;	
;	==============
;	Processor:		Microchip PIC 16XXX
;	Clock:			4.00 MHz XT
;	Instruction cycle:	1 MIPS
;	Additional hardware:	MAX232
;
;
;	List of macros:
;
;		"RS232init"	- Initialization
;		"SEND 'c' "	- send ASCII charcters
;		"SENDw"		- send dump of w register
;		"RECEIVE"	- Function in ISR, receive data in RXD register
;
;	In the main program you must have the next declaration:
;	=======================================================
;	Cblock 0x0C	; Beginning of RAM
;       RS_TEMP1	; Temp register
;       RS_TEMP2
;       fifo_char
;       fifo_len
;       fifo_start
;       fifo_base
;       fifo_buffer:16
;
; TXD   ; TX-Data register
;	RXD		; RX-Data register
;	endc
;
;	#define	TXport	PORTA,0x00	; RS232 TX pin 
;	#define	TXtris	TRISA,0x00	
;
;	ORG     0x04			; interrupt vector
;	goto	ISR			; jump on ISR (Interrupt Service Routine)
;
;
;	;***** INTERRUPT SERVICE ROUTINE *****
;
;	ISR	bcf	INTCON,GIE	; Disable all interrupts
;		btfsc	INTCON,GIE	
;		goto	ISR
;		RECEIVE
;		<... future code ...>
;	ISRend	bcf	INTCON,INTF	; Erase interrupt flag RB0/INT
;		RETFIE			; Enable interrupts - INTCON,GIE
;       
;*********************************************************************************

;***** Declaring the hardware *****	

	#define	RXport PORTB,0	
	#define	RXtris TRISB,0	
	
;***** Declaring constants *****

	CONSTANT LF  =	d'10'	; Line Feed
	CONSTANT CR  =	d'13'	; Carriage Return
	CONSTANT TAB =	d'9'	; Tabulator
	CONSTANT BS  =	d'8'	; Backspace

;***** Macros *****

RS232init macro
	call	RS_init
	endm


;***** Subprograms *****

RS_init
	BANK1
	clrf	OPTION_REG
	bsf	RXtris		
	BANK0
  clrf  fifo_start
  clrf  fifo_len
  movlw fifo_buffer
  movwf fifo_base
;
  movlw b'10010000'
	movwf	INTCON	
	RETURN


S_Wait	movlw	0x1E		
        movwf   RS_TEMP2	
	goto	X_Wait		
	
Rs_Wait movlw 0x02
        movwf   RS_TEMP2	
	goto	X_Wait		

R_Wait  movlw 0x0c
        movwf   RS_TEMP2	
	goto	X_Wait

Rf_Wait  movlw 0x08
        movwf   RS_TEMP2	
	goto	X_Wait

X_Wait  decfsz  RS_TEMP2,1
	goto	X_Wait
	RETURN

RECEIVE	macro
RECsub	call	Rs_Wait
	btfsc	RXport
  goto	ISRend
  movlw 0x08
  movwf   RS_TEMP1
	goto	RECa		
RECa	call	R_Wait
	btfss	RXport
	goto	RECb
	bsf	RXD,0x07	
	goto	RECc
RECb	bcf	RXD,0x07	
RECc    decfsz  RS_TEMP1,0	
	rrf	RXD,1		
  decfsz  RS_TEMP1,1
	goto	RECa
	call	Rf_Wait
;
  movfw fifo_len
  xorlw 0x10
  btfsc STATUS,Z
  goto	ISRend
  movfw fifo_start
  addwf fifo_len,w
  andlw 0x0f
  addwf fifo_base,w
  movwf FSR
  movfw RXD
  movwf INDF
  incf  fifo_len,f
;
  btfss RXport
  decf  fifo_len,f
	goto	ISRend
	endm


FIFOget
  movfw fifo_len
  btfsc STATUS,Z
  return
  movfw  fifo_base
  bcf	INTCON,GIE	;lock out any interrupts for a short tim
  bcf	INTCON,GIE	;just to be shure
  addwf fifo_start,w
  movwf FSR
  incf  fifo_start,w
  andlw 0x0f
  movwf fifo_start
  bsf	INTCON, GIE	; and enable again
  decf  fifo_len,f
  movf  INDF,w
  movwf fifo_char
  return



