;*********************************************************************************
;
;       Wait routines for PIC 16XXX
;       ===========================
;
;	==============
;	Processor:		Microchip PIC 16XXX
;	Clock:			4.00 MHz XT
;	Instruction cycle:	1 MIPS
;
;       Prescaler values:
;
;               b'00000000' == 1:2      ==>  0.512 ms
;               b'00000001' == 1:4      ==>  1.02 ms
;               b'00000111' == 1:256    ==>  65.3 ms
;
;               Waittime = WCYCLE (= timeconst) * prescaler
;
;	List of macros:
;
;               "WAIT xxx"              ; standard
;               "WAITX xxx, xxx"        ; extended with prescaler value
;
;	In the main program you must have the next declaration:
;	=======================================================
;	Cblock 0x0C	; Beginning of RAM
;	WCYCLE
;	PRESCwait
;       endc
;
;*********************************************************************************


;***** Declaring constants *****

        CONSTANT PRESCstd = b'00000001' ; standard prescaler value for TMR0

;***** Macros *****

WAIT    macro   timeconst_1
        movlw   timeconst_1
        call    WAITstd
        endm

WAITX   macro   timeconst_2, PRESCext
        movlw   timeconst_2
        movwf   WCYCLE          ; set the delay time period
        movlw   PRESCext        ; write specific prescaler value
        call    WAIT_x
        endm

;***** Subprograms *****

WAITstd movwf   WCYCLE          ; set the delay time period
        movlw   PRESCstd        ; write specific prescaler value
WAIT_x  clrf    TMR0
        BANK1
        movwf   OPTION_REG      ; assing the prescaler to TMR0 timer
        BANK0

WAITa   bcf     INTCON,T0IF     ; erase TMR0 Overflow Flag
WAITb   btfss   INTCON,T0IF     ; Check whether it is erased, skip if it isn`t
        goto    WAITb           ; Wait loop
        decfsz  WCYCLE,1        ; Repeat the loop if delay period has not run out
        goto    WAITa            
        RETURN
